//+------------------------------------------------------------------+
//|                                                  MACD arrows.mq4 |
//|                        Copyright 2016, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2016, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property strict
//---- indicator settings
#property  indicator_separate_window
#property  indicator_buffers 5
#property  indicator_color1  Green
#property  indicator_color2  Red
#property  indicator_color3  Silver

#property  indicator_width1  3
#property  indicator_width2  3
//---- indicator parameters
extern int FastEMA=5;
extern int SlowEMA=13;
extern int SignalSMA=1;
extern int Arrow_Gap= 3;
extern bool Audible_Alerts=true;
extern bool Push_Notifications=false;
//---- indicator buffers
double     MacdBuffer[],MacdBufferUp[],MacdBufferDown[];
double     SignalBuffer[];
double     BuyBuffer[],SellBuffer[];
double myPoint; //initialized in OnInit
datetime time_alert; //used when sending alert
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   DeleteArrows();
//---- drawing settings
   IndicatorBuffers(6);

   SetIndexBuffer(0,MacdBufferUp);
   SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexEmptyValue(0,0.0);
   SetIndexBuffer(1,MacdBufferDown);
   SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexEmptyValue(1,0.0);
   SetIndexBuffer(2,SignalBuffer);
   SetIndexStyle(2,DRAW_LINE);
   SetIndexDrawBegin(2,SignalSMA);
   SetIndexBuffer(3,BuyBuffer);
   SetIndexStyle(3,DRAW_NONE);
   SetIndexEmptyValue(3,0.0);
   SetIndexBuffer(4,SellBuffer);
   SetIndexStyle(4,DRAW_NONE);
   SetIndexEmptyValue(4,0.0);
   SetIndexBuffer(5,MacdBuffer);
   SetIndexStyle(5,DRAW_NONE);

   SetIndexLabel(0,"Bull Buffer");
   SetIndexLabel(1,"Bear Buffer");
   SetIndexLabel(2,"Signal");
   SetIndexLabel(3,"Buy");
   SetIndexLabel(4,"Sell");

   IndicatorDigits(Digits+3);
   IndicatorShortName("MACD_color("+FastEMA+","+SlowEMA+","+SignalSMA+")");
   myPoint=Point();
   if(Digits()==5 || Digits()==3)
     {
      myPoint*=10;
     }
   return(0);
  }
//+------------------------------------------------------------------+
//| Moving Averages Convergence/Divergence                           |
//+------------------------------------------------------------------+
int start()
  {
   int limit,i;
   int counted_bars=IndicatorCounted();
//---- last counted bar will be recounted
   limit=(counted_bars<1)?Bars-100:Bars-counted_bars;
//---- macd counted in the 1-st buffer
   for(i=limit; i>=0; i--)
      MacdBuffer[i]=iMA(NULL,0,FastEMA,0,MODE_EMA,PRICE_CLOSE,i)-iMA(NULL,0,SlowEMA,0,MODE_EMA,PRICE_CLOSE,i);
//---
   for(i=limit; i>=0; i--)
     {
      MacdBufferDown[i]=0.0; MacdBufferUp[i]=0.0;

      if(i>=1) //??? ????? ?? ??????????? ???????? ???? //break the null bar  
        {
         if(MacdBuffer[i+1]-MacdBuffer[i]>=0)
            MacdBufferDown[i]=MacdBuffer[i]; //??????? ???????? //condition of color
         else 
            MacdBufferUp[i]=MacdBuffer[i];
        }

      //Test 3 Bar rule
      int ia=0;
      int ix;
      int iGreen=0;
      if(MacdBufferDown[i+1]!=0 && MacdBufferDown[i+2]!=0 && MacdBufferDown[i+3]!=0 && MacdBufferDown[i+4]==0)
        {
         for(ix=5;ix<50;ix++)
            if(MacdBufferDown[i+ix]!=iGreen)
               break;
//---
         if(ix>=7)
           {
            ia=0;
            for(ix=ix;ix<50;ix++)
              {
               if(MacdBufferDown[i+ix]==iGreen)
                  break;
               ia++;
              }

            if(ia>=3)
              {
               double high=High[i]+(Arrow_Gap*myPoint);
               SellBuffer[i]=high;
               if(ObjectFind("ArrowSell"+string(i))==0)ObjectDelete("ArrowSell"+string(i));
               ObjectCreate(0,"ArrowSell"+string(i),OBJ_ARROW_DOWN,0,iTime(NULL,0,i),high);//Draw Arrow Down
               ObjectSet("ArrowSell"+string(i),OBJPROP_STYLE,STYLE_SOLID);
               ObjectSet("ArrowSell"+string(i),OBJPROP_ARROWCODE,SYMBOL_ARROWDOWN);
               ObjectSet("ArrowSell"+string(i),OBJPROP_COLOR,Red);
               if(i==0 && Time[0]!=time_alert)
                 { 
                  myAlert("Sell Alert "+string(i));
                  time_alert=Time[0];
                 }
              }
           }
        }
//---
      if(MacdBufferDown[i+1]==iGreen && MacdBufferDown[i+2]==iGreen && MacdBufferDown[i+3]==iGreen && MacdBufferDown[i+4]!=iGreen)
        {
         for(ix=5;ix<50;ix++)
            if(MacdBufferDown[i+ix]==iGreen)
               break;

         if(ix>=7)
           {
            ia=0;
            for(ix=ix;ix<50;ix++)
              {

               if(MacdBufferDown[i+ix]!=iGreen)
                  break;
               ia++;
              }

            if(ia>=3)
              {
               double low=Low[i]-(Arrow_Gap*myPoint);
               BuyBuffer[i]=low;
               ObjectCreate(0,"ArrowBuy"+string(i),OBJ_ARROW_UP,0,iTime(NULL,0,i),low);//Draw Arrow Up       
               ObjectSet("ArrowBuy"+string(i),OBJPROP_STYLE,STYLE_SOLID);
               ObjectSet("ArrowBuy"+string(i),OBJPROP_ARROWCODE,SYMBOL_ARROWUP);
               ObjectSet("ArrowBuy"+string(i),OBJPROP_COLOR,Green);
               if(i==0 && Time[0]!=time_alert)
                 { 
                  myAlert("Buy Alert "+string(i));
                  time_alert=Time[0];
                 }
              }
           }
        }
     }
   for(i=0; i<limit; i++) 
      SignalBuffer[i]=iMAOnArray(MacdBuffer,Bars,SignalSMA,0,MODE_SMA,i);
   return(0);
  }
//+------------------------------------------------------------------+

void DeleteArrows()
  {
   for(int i=ObjectsTotal()-1; i>-1; i--)
      if(StringFind(ObjectName(i),"Arrow")>=0) ObjectDelete(ObjectName(i));
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   DeleteArrows();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void myAlert(string message)
  {
//---
   if(Audible_Alerts) 
      Alert("MACD Color Signal @ "+Symbol()+","+(string)Period()+" | "+message);
   if(Push_Notifications) 
      SendNotification("MACD Color Signal @ "+Symbol()+","+(string)Period()+" | "+message);
  }
//+------------------------------------------------------------------+
